@@ -170,17 +170,14 @@ pdl_error pdl_allocdata(pdl *it) {
 }
 
 pdl* pdl_pdlnew() {
-     pdl* it;
-     it = (pdl*) malloc(sizeof(pdl));
+     pdl *it = (pdl*) malloc(sizeof(pdl));
      if (!it) return it;
      memset(it, 0, sizeof(pdl));
      it->magicno = PDL_MAGICNO;
      it->datatype = PDL_D;
      it->trans_parent = NULL;
      it->vafftrans = NULL;
-     it->sv = NULL;
-     it->datasv = 0;
-     it->data = 0;
+     it->data = it->datasv = it->sv = NULL;
      it->has_badvalue = 0;
      it->state = PDL_NOMYDIMS;
      it->dims = it->def_dims;
@@ -251,36 +248,35 @@ pdl_error pdl__free(pdl *it) {
     PDLDEBUG_f(printf("pdl__free %p\n",(void*)it));
     PDL_CHKMAGIC(it);
     /* now check if magic is still there */
-    if (pdl__ismagic(it)) {
+    if (pdl__ismagic(it))
       PDLDEBUG_f(printf("%p is still magic\n",(void*)it);pdl__print_magic(it));
-    }
     it->magicno = 0x42424245;
-    if(it->dims       != it->def_dims)       free((void*)it->dims);
-    if(it->dimincs    != it->def_dimincs)    free((void*)it->dimincs);
-    if(it->broadcastids  != it->def_broadcastids)  free((void*)it->broadcastids);
-    if(it->vafftrans) {
+    if (it->dims       != it->def_dims)       free((void*)it->dims);
+    if (it->dimincs    != it->def_dimincs)    free((void*)it->dimincs);
+    if (it->broadcastids  != it->def_broadcastids)  free((void*)it->broadcastids);
+    if (it->vafftrans) {
 	pdl_vafftrans_free(it);
     }
     pdl_trans_children *p1 = it->trans_children.next;
-    while(p1) {
+    while (p1) {
 	pdl_trans_children *p2 = p1->next;
 	free(p1);
 	p1 = p2;
     }
 /* Call special freeing magic, if exists */
-    if(PDL_ISMAGIC(it)) {
+    if (PDL_ISMAGIC(it)) {
 	pdl__call_magic(it, PDL_MAGIC_DELETEDATA);
 	pdl__magic_free(it);
     }
-    if(it->datasv) {
-	    PDLDEBUG_f(printf("SvREFCNT_dec datasv=%p\n",it->datasv);)
+    if (it->datasv) {
+	    PDLDEBUG_f(printf("SvREFCNT_dec datasv=%p\n",it->datasv));
 	    SvREFCNT_dec(it->datasv);
 	    it->data=0;
-    } else if(it->data && it->data != &it->value) {
+    } else if (it->data && it->data != &it->value) {
 	    pdl_pdl_warn("Warning: special data without datasv is not freed currently!!");
     }
     if(it->hdrsv) {
-	PDLDEBUG_f(printf("SvREFCNT_dec hdrsv=%p\n",it->hdrsv);)
+	PDLDEBUG_f(printf("SvREFCNT_dec hdrsv=%p\n",it->hdrsv));
 	SvREFCNT_dec(it->hdrsv);
 	it->hdrsv = 0;
     }
@@ -620,8 +616,10 @@ pdl_error pdl__addchildtrans(pdl *it,pdl_trans *trans)
 	return PDL_err;
 }
 
-pdl_error pdl_make_physdims(pdl *it) {
+pdl_error pdl__make_physdims_recprotect(pdl *it, int recurse_count) {
   pdl_error PDL_err = {0, NULL, 0};
+  if (recurse_count > 1000)
+    return pdl_make_error_simple(PDL_EUSERERROR, "PDL:Internal Error: data structure recursion limit exceeded (max 1000 levels)\n\tThis could mean that you have found an infinite-recursion error in PDL, or\n\tthat you are building data structures with very long dataflow dependency\n\tchains.  You may want to try using sever() to break the dependency.\n");
   if (!it) return pdl_make_error_simple(PDL_EFATAL, "make_physdims called with NULL");
   PDLDEBUG_f(printf("make_physdims %p state=", it);pdl_dump_flags_fixspace(it->state, 0, PDL_FLAGS_PDL));
   PDL_CHKMAGIC(it);
@@ -634,12 +632,15 @@ pdl_error pdl_make_physdims(pdl *it) {
   PDL_Indx i;
   for (i=0; i<trans->vtable->nparents; i++)
     if (trans->pdls[i]->state & PDL_PARENTDIMSCHANGED)
-      PDL_RETERROR(PDL_err, pdl_make_physdims(trans->pdls[i]));
+      PDL_RETERROR(PDL_err, pdl__make_physdims_recprotect(trans->pdls[i], recurse_count+1));
   PDLDEBUG_f(printf("make_physdims: calling redodims trans=%p on pdl=%p\n", trans, it));
   REDODIMS(PDL_RETERROR, trans);
   PDLDEBUG_f(printf("make_physdims exit pdl=%p\n", it));
   return PDL_err;
 }
+pdl_error pdl_make_physdims(pdl *it) {
+  return pdl__make_physdims_recprotect(it, 0);
+}
 
 static inline pdl_error pdl_trans_flow_null_checks(pdl_trans *trans, int *ret) {
   pdl_error PDL_err = {0, NULL, 0};
@@ -700,13 +701,18 @@ pdl_error pdl_make_trans_mutual(pdl_trans *trans)
   for (i=0; i<nparents; i++) {
     pdl *parent = pdls[i];
     PDL_RETERROR(PDL_err, pdl__addchildtrans(parent,trans));
-    if (parent->state & PDL_DATAFLOW_F) trans->flags |= PDL_ITRANS_DO_DATAFLOW_F;
+    if (parent->state & PDL_DATAFLOW_F) {
+      if (!(parent->trans_parent && (parent->trans_parent->flags & PDL_ITRANS_DO_DATAFLOW_F))) /* not inherently flowing */
+        parent->state &= ~PDL_DATAFLOW_F;
+      trans->flags |= PDL_ITRANS_DO_DATAFLOW_F;
+      dataflow = 1;
+    }
   }
   char wasnull[npdls];
   for (i=nparents; i<npdls; i++) {
     pdl *child = pdls[i];
     wasnull[i] = !!(child->state & PDL_NOMYDIMS);
-    PDLDEBUG_f(printf("make_trans_mutual child=%p wasnull[%"IND_FLAG"]=%d", child, i, (int)wasnull[i]));
+    PDLDEBUG_f(printf("make_trans_mutual child=%p wasnull[%"IND_FLAG"]=%d\n", child, i, (int)wasnull[i]));
     if (dataflow) {
       /* This is because for "+=" (a = a + b) we must check for
          previous parent transformations and mutate if they exist
@@ -720,16 +726,19 @@ pdl_error pdl_make_trans_mutual(pdl_trans *trans)
       child->state = (child->state & ~PDL_NOMYDIMS) | PDL_MYDIMS_TRANS;
   }
   if (!dataflow) {
-    PDL_ACCUMERROR(PDL_err, pdl__ensure_trans(trans, dataflow ? 0 : PDL_PARENTDIMSCHANGED, 0, 0));
-    for (i=vtable->nparents; i<vtable->npdls; i++) {
-      pdl *child = trans->pdls[i];
-      char isvaffine = !!PDL_VAFFOK(child);
-      PDLDEBUG_f(printf("make_trans_mutual isvaffine=%d wasnull=%d\n", (int)isvaffine, (int)wasnull[i]));
-      if (!isvaffine || wasnull[i])
-        CHANGED(child, wasnull[i] ? PDL_PARENTDIMSCHANGED : PDL_PARENTDATACHANGED, 0);
-      if (isvaffine)
-        CHANGED(child->vafftrans->from,PDL_PARENTDATACHANGED,0);
-    }
+    PDL_ACCUMERROR(PDL_err, pdl__ensure_trans(trans, PDL_PARENTDIMSCHANGED, 0, 0));
+    if (PDL_err.error)
+      PDLDEBUG_f(printf("make_trans_mutual got error in ensure, not calling CHANGED on children\n"));
+    else
+      for (i=vtable->nparents; i<vtable->npdls; i++) {
+        pdl *child = trans->pdls[i];
+        char isvaffine = !!PDL_VAFFOK(child);
+        PDLDEBUG_f(printf("make_trans_mutual isvaffine=%d wasnull=%d\n", (int)isvaffine, (int)wasnull[i]));
+        if (!isvaffine || wasnull[i])
+          CHANGED(child, wasnull[i] ? PDL_PARENTDIMSCHANGED : PDL_PARENTDATACHANGED, 0);
+        if (isvaffine)
+          CHANGED(child->vafftrans->from,PDL_PARENTDATACHANGED,0);
+      }
     PDL_ACCUMERROR(PDL_err, pdl_destroytransform(trans,0,0));
   }
   PDLDEBUG_f(printf("make_trans_mutual exit %p\n",(void*)trans));
@@ -819,7 +828,6 @@ pdl_error pdl_changed(pdl *it, int what, int recursing) {
     printf("pdl_changed: entry for pdl %p recursing: %d, what=",
            (void*)it,recursing);
     pdl_dump_flags_fixspace(what,0,PDL_FLAGS_PDL);
-    if (it->state & PDL_TRACEDEBUG) pdl_dump(it);
   );
   pdl_trans *trans = it->trans_parent;
   if (recursing) {
@@ -830,7 +838,8 @@ pdl_error pdl_changed(pdl *it, int what, int recursing) {
     ) ? what : what & ~PDL_PARENTDATACHANGED;
     if (pdl__ismagic(it)) pdl__call_magic(it,PDL_MAGIC_MARKCHANGED);
   }
-  if (trans && !recursing && (trans->flags & PDL_ITRANS_DO_DATAFLOW_B)) {
+  if (trans && !recursing && (trans->flags & PDL_ITRANS_DO_DATAFLOW_B)
+      && (what & PDL_PARENTDATACHANGED)) {
     if (it->vafftrans) {
       if (it->state & PDL_ALLOCATED) {
         PDLDEBUG_f(printf("pdl_changed: calling writebackdata_vaffine (pdl %p)\n",(void*)it));
@@ -873,7 +882,7 @@ pdl_error pdl__make_physvaffine_recprotect(pdl *it, int recurse_count)
   pdl_error PDL_err = {0, NULL, 0};
   PDL_Indx i,j;
   PDLDEBUG_f(printf("make_physvaffine %p\n",(void*)it));
-  PDL_RETERROR(PDL_err, pdl_make_physdims(it));
+  PDL_RETERROR(PDL_err, pdl__make_physdims_recprotect(it, recurse_count+1));
   if (!it->trans_parent || !(it->trans_parent->flags & PDL_ITRANS_ISAFFINE)) {
     PDLDEBUG_f(printf("make_physvaffine handing off to make_physical %p\n",(void*)it));
     return pdl__make_physical_recprotect(it, recurse_count+1);
