@@ -16,7 +16,8 @@ our $pars_re = qr/^
 	\s*(?:($complex_regex|$typeregex)\b([+]*)|)\s*	# $1,2: first option then plus
 	(?:$sqbr_re)?\s*	# $3: The initial [option] part
 	(\w+)			# $4: The name
-	\(([^)]*)\)		# $5: The indices
+	\((.*)\)		# $5: The indices
+	\s*\Z			# that's all
 /x;
 my %flag2info = (
   io => [[qw(FlagW)]],
@@ -41,37 +42,48 @@ my %flag2c = qw(
   FlagPhys PDL_PARAM_ISPHYS
   FlagIgnore PDL_PARAM_ISIGNORE
 );
+my $calc_re = qr{
+  (\w+)\s*=\s*      # paren group 1 (dim name) - from perlre/PARNO
+  CALC
+  (                 # paren group 2 (parens)
+    \(
+      (             # paren group 3 (contents of parens)
+      (?:
+       (?> [^()]+ ) # Non-parens without backtracking
+      |
+       (?2)         # Recurse to start of paren group 2
+      )*
+      )
+    \)
+  )
+}xo;
 sub new {
-	my($type,$string,$badflag,$sig) = @_;
-	$badflag ||= 0;
-	my $this = bless {Number => "PDL_UNDEF_NUMBER", BadFlag => $badflag, Sig => $sig},$type;
-	# Parse the parameter string. Note that the regexes for this match were
-	# originally defined here, but were moved to PDL::PP for FullDoc parsing.
-	$string =~ $pars_re
-		 or confess "Invalid pdl def $string (regex $pars_re)\n";
-	my($opt1,$opt_plus,$sqbr_opt,$name,$inds) = map $_ // '', $1,$2,$3,$4,$5;
-	print "PDL: '$opt1$opt_plus', '$sqbr_opt', '$name', '$inds'\n"
-		  if $::PP_VERBOSE;
-	croak "Invalid Pars name: $name"
-	  if $INVALID_PAR{$name};
+  my ($type,$string,$badflag,$sig) = @_;
+  $badflag ||= 0;
+  my $this = bless {Number => "PDL_UNDEF_NUMBER", BadFlag => $badflag, Sig => $sig},$type;
+  $string =~ $pars_re or croak "pp_def($this->{Sig}{OpName}): Invalid pdl def $string (regex $pars_re)\n";
+  my($opt1,$opt_plus,$sqbr_opt,$name,$inds) = map $_ // '', $1,$2,$3,$4,$5;
+  print "PDL: '$opt1$opt_plus', '$sqbr_opt', '$name', '$inds'\n"
+    if $::PP_VERBOSE;
+  croak "pp_def($this->{Sig}{OpName}): Invalid Pars name: $name" if $INVALID_PAR{$name};
 # Set my internal variables
-	$this->{Name} = $name;
-	$this->{Flags} = [(split ',',$sqbr_opt),($opt1?$opt1:())];
-	for(@{$this->{Flags}}) {
-		confess("Invalid flag $_ given for $string\n")
-			unless my ($set, $store) = @{ $flag2info{$_} || [] };
-		$this->{$store} = $_ if $store;
-		$this->{$_} = 1 for @$set;
-	}
-	if ($this->{FlagTyped} && $opt_plus) {
-	  $this->{FlagTplus} = 1;
-	}
-	$this->{Type} &&= PDL::Type->new($this->{Type});
-	$this->{RawInds} = [map{
-		s/\s//g; 		# Remove spaces
-		$_;
-	} split ',', $inds];
-	return $this;
+  $this->{Name} = $name;
+  $this->{Flags} = [(split ',',$sqbr_opt),($opt1?$opt1:())];
+  for(@{$this->{Flags}}) {
+    croak("pp_def($this->{Sig}{OpName}): Invalid flag $_ given for $string\n")
+      unless my ($set, $store) = @{ $flag2info{$_} || [] };
+    $this->{$store} = $_ if $store;
+    $this->{$_} = 1 for @$set;
+  }
+  $this->{FlagTplus} = 1 if $this->{FlagTyped} && $opt_plus;
+  $this->{Type} &&= PDL::Type->new($this->{Type});
+  $this->{Ind2Calc} = \my %ind2calc;
+  $ind2calc{$1} = $3 while $inds =~ s#$calc_re#$1#;
+  $this->{RawInds} = [map{
+    s/\s//g; 		# Remove spaces
+    $_;
+  } split ',', $inds];
+  return $this;
 }
 
 sub cflags {
@@ -79,11 +91,11 @@ sub cflags {
   map $flag2c{$_}, grep $this->{$_}, sort keys %flag2c;
 }
 
-sub name {return (shift)->{Name}}
+sub name {$_[0]{Name}}
 
 sub add_inds {
 	my($this,$dimsobj) = @_;
-	$this->{IndObjs} = [my @objs = map $dimsobj->get_indobj_make($_), @{$this->{RawInds}}];
+	$this->{IndObjs} = [my @objs = map $dimsobj->get_indobj_make($_, $this->{Ind2Calc}{$_}), @{$this->{RawInds}}];
 	my %indcount;
 	$this->{IndCounts} = [ map 0+($indcount{$_->name}++), @objs ];
 	$this->{IndTotCounts} = [ map $indcount{$_->name}, @objs ];
@@ -124,7 +136,7 @@ sub finalcheck {
 sub getcreatedims {
   my $this = shift;
   return map
-    { croak "can't create: index size ".$_->name." not initialised"
+    { croak "pp_def($this->{Sig}{OpName}): can't create: index size ".$_->name." not initialised"
 	if !defined($_->{Value}) || $_->{Value} < 1;
       $_->{Value} } @{$this->{IndObjs}};
 }
@@ -151,9 +163,9 @@ sub get_substname {
 }
 
 sub get_incname {
-	my($this,$ind,$for_local) = @_;
-	return "inc_sizes[PDL_INC_ID(__privtrans->vtable,$this->{Number},$ind)]" if !$for_local;
-	"__inc_$this->{Name}_".$this->get_substname($ind);
+  my($this,$ind,$for_local) = @_;
+  return "inc_sizes[PDL_INC_ID(__privtrans->vtable,$this->{Number},$ind)]" if !$for_local;
+  "__inc_$this->{Name}_".$this->get_substname($ind);
 }
 
 sub get_incregisters {
@@ -168,22 +180,25 @@ sub get_incregisters {
 
 # Print an access part.
 sub do_access {
-	my($this,$inds,$context) = @_;
-	my $pdl = $this->{Name};
-# Parse substitutions into hash
-	my %subst = map
-	 {/^\s*(\w+)\s*=>\s*(\S*)\s*$/ or confess "Invalid subst $_ in ($inds) (no spaces in => value)\n"; ($1,$2)}
-		PDL::PP::Rule::Substitute::split_cpp($inds);
-# Generate the text
-	my $text = "(${pdl}_datap)[" . join('+','0', map
-		$this->do_indterm($pdl,$_,\%subst,$context),
-	0..$#{$this->{IndObjs}}) . "]";
-# If not all substitutions made, the user probably made a spelling
-# error. Barf.
-	if(scalar(keys %subst) != 0) {
-		confess("Substitutions left: ".(join ',',sort keys %subst)."\n");
-	}
-	$text;
+  my($this,$inds,$context) = @_;
+  my $pdl = $this->{Name};
+  my %subst = map {
+    if (!/^\s*(\w+)\s*=>\s*(\S*)\s*$/) {
+      my $msg = "Invalid subst '$_' in \$$pdl($inds):";
+      $msg .= " no '=>' seen" if !/=>/;
+      $msg .= " invalid dim name '$1'" if /^\s*([^\w]*?)\s*=>/;
+      $msg .= " (no spaces in => value)" if /=>\s*\S\s*\S/;
+      croak "pp_def($this->{Sig}{OpName}): $msg\n";
+    }
+    ($1,$2)
+  } PDL::PP::Rule::Substitute::split_cpp($inds);
+  my $text = "(${pdl}_datap)[" .
+    join('+','0', map $this->do_indterm($pdl,$_,\%subst,$context), 0..$#{$this->{IndObjs}})
+  . "]";
+  # If not all substitutions made, the user probably made a spelling error
+  croak "pp_def($this->{Sig}{OpName}): Substitutions left for \$$pdl($inds): ".(join ',',sort keys %subst)."\n"
+    if keys(%subst) != 0;
+  $text;
 }
 
 sub do_pdlaccess {
@@ -202,7 +217,7 @@ sub do_indterm { my($this,$pdl,$ind,$subst,$context) = @_;
   my $index = delete($subst->{$substname}) //
 # No => get the one from the nearest context.
     (grep $_ eq $substname, map $_->[1], reverse @$context)[0];
-  confess "Access Index not found: $pdl, $ind, @{[$this->{IndObjs}[$ind]->name]}
+  croak "pp_def($this->{Sig}{OpName}): Access Index not found: $pdl, $ind, @{[$this->{IndObjs}[$ind]->name]}
 	  On stack:".(join ' ',map {"($_->[0],$_->[1])"} @$context)."\n"
 	  if !defined $index;
   return "(".($this->get_incname($ind,1))."*($index))";
