@@ -4,9 +4,9 @@ use PDL;  # this must be called before (!) 'use Inline Pdlpp' calls
 use PDL::NiceSlice; # only used to demonstrate how to switch off below
 use Inline Pdlpp; # the actual code is in the __Pdlpp__ block below
 
-$a = sequence 10;
-print $a(0:4),"\n";
-print $a->inc->(0:4),"\n";
+$x = sequence 10;
+print $x(0:4),"\n";
+print $x->inc->(0:4),"\n";
 
 # important!
 no PDL::NiceSlice; # disable NiceSlice before(!) the data section
