@@ -1,8 +1,10 @@
 package PDL::Graphics2D;
 
+use strict;
+use warnings;
 use Exporter 'import'; # gives you Exporter's import() method directly
-@EXPORT = qw(imag2d imag2d_update twiddle);     # symbols to export on request
-@EXPORT_OK = qw(imag2d imag2d_update twiddle);  # symbols to export on request
+our @EXPORT = qw(imag2d imag2d_update twiddle);     # symbols to export on request
+our @EXPORT_OK = qw(imag2d imag2d_update twiddle);  # symbols to export on request
 
 =head1 NAME
 
@@ -194,7 +196,7 @@ sub mouse_click {
    # search for image corresponding to window
    foreach my $entry ( @imag2d_list ) {
       if ( $entry->{window_id} == $window_id ) {
-         $img = $entry->{img};  # 2D piddle for now
+         $img = $entry->{img};  # 2D ndarray for now
          last;
       }
    }
@@ -249,7 +251,7 @@ sub resize_window {
    # search for image corresponding to window
    foreach my $entry ( @imag2d_list ) {
       if ( $entry->{window_id} == $window_id ) {
-         $img = $entry->{img};  # 2D piddle for now
+         $img = $entry->{img};  # 2D ndarray for now
          last;
       }
    }
@@ -323,7 +325,7 @@ sub display_image {
    # search for image corresponding to window
    foreach my $entry ( @imag2d_list ) {
       if ( $entry->{window_id} == $window_id ) {
-         $img = $entry->{img};  # 2D piddle for now
+         $img = $entry->{img};  # 2D ndarray for now
          last;
       }
    }
@@ -399,7 +401,7 @@ sub ModeMenu {
       # search for image corresponding to window
       foreach my $listentry ( @imag2d_list ) {
          if ( $listentry->{window_id} == $window_id ) {
-            $img = $listentry->{img};  # 2D piddle for now
+            $img = $listentry->{img};  # 2D ndarray for now
             last;
          }
       }
@@ -591,7 +593,7 @@ sub display_new_window {
 };
 
 #------------------------------------------------------------------------
-# Display piddle as 2-D image in window using OpenGL
+# Display ndarray as 2-D image in window using OpenGL
 #------------------------------------------------------------------------
 
 =head1 FUNCTIONS
@@ -613,7 +615,7 @@ $c==4 the display is in GL_RGBA, for $c==3 the display is GL_RGB,
 for $c==2 the display is GL_LUMINANCE_ALPHA, and for $c==1 or for
 for dimensions ($M,$N) then the display is GL_LUMINANCE.
 
-This routine does not yet thread but multiple images may be
+This routine does not yet broadcast but multiple images may be
 viewed at the same time in separate windows by multiple
 calls to imag2d().  TriD graphics visualization windows and the
 imag2d() windows may be created and used independently.
@@ -626,12 +628,12 @@ the sub, only the imag2d() windows will update correctly.
 
   $window_id = imag2d($image, $name, $zoom, $x_off, $y_off);
     
-    creates a new image figure window from the input piddle
+    creates a new image figure window from the input ndarray
     with the given title, zoom factor, and position (if possible)
     
     $window_id - may be used to refer to the figure window
     
-    $image - 2D image piddle with at least 2 or 3 dimensions
+    $image - 2D image ndarray with at least 2 or 3 dimensions
              e.g. [M,N], [1,M,N], [2,M,N], [3,M,N], [4,M,N]
     
     $name - the name to use for the figure window (optional)
@@ -659,7 +661,7 @@ the sub, only the imag2d() windows will update correctly.
 
 =for ref
 
-Update an existing imag2d window with new piddle data
+Update an existing imag2d window with new ndarray data
 
 =for usage
 
@@ -824,7 +826,7 @@ sub imag2d_update {
    # search for image corresponding to window
    foreach my $entry ( @imag2d_list ) {
       if ( $entry->{window_id} == $win_id ) {
-         $img = $entry->{img};  # 2D piddle for now
+         $img = $entry->{img};  # 2D ndarray for now
          last;
       }
    }
@@ -914,10 +916,10 @@ sub twiddle {
 }
 
 #------------------------------------------------------------------------
-# Threaded image display as tiles (code from PDL::Graphics::TriD::Image)
+# Broadcasted image display as tiles (code from PDL::Graphics::TriD::Image)
 #------------------------------------------------------------------------
 
-# N-D piddle -> 2-D
+# N-D ndarray -> 2-D
 sub flatten {
    my ($this,$bin_align) = @_;
 
@@ -975,8 +977,6 @@ sub flatten {
    my $firsty = 1;
    my $spi;
    for (@dims[reverse(2..$#dims)]) {
-      $foop->make_physdims();
-      # print "FOOP: \n"; $foop->dump;
       if ($ind % 2 == 0) {
          $spi = $foop->getdim(1)/$_;
          $foop = $foop->splitdim(1,$spi)->slice(":,0:-2")->mv(2,3);
