@@ -6,8 +6,11 @@
 
 use strict;
 use warnings;
-use OpenGL qw(:all);
+no warnings 'redefine';
+use OpenGL qw/ :glfunctions :glconstants gluPerspective gluOrtho2D /;
+use OpenGL::GLUT qw( :all );
 use PDL::Graphics::OpenGL::Perl::OpenGL;
+use PDL::Core qw(barf);
 
 $PDL::Graphics::TriD::create_window_sub = # warnings
 $PDL::Graphics::TriD::create_window_sub = sub {
@@ -27,6 +30,7 @@ sub PDL::Graphics::TriD::Material::togl{
 }
 
 $PDL::Graphics::TriD::any_cannots = 0;
+$PDL::Graphics::TriD::verbose //= 0;
 
 sub PDL::Graphics::TriD::Object::cannot_mklist {
 	return 0;
@@ -34,108 +38,73 @@ sub PDL::Graphics::TriD::Object::cannot_mklist {
 
 sub PDL::Graphics::TriD::Object::gl_update_list {
   my($this) = @_;
-  if($this->{List}) {
-	 glDeleteLists($this->{List},1);
-  }
-  my $lno = glGenLists(1);
-  $this->{List} = $lno;
+  glDeleteLists($this->{List},1) if $this->{List};
+  $this->{List} = my $lno = glGenLists(1);
   print "GENLIST $lno\n" if($PDL::Graphics::TriD::verbose);
   glNewList($lno,GL_COMPILE);
-  if ($PDL::Graphics::TriD::any_cannots) {
-	 for(@{$this->{Objects}}) {
-		if(!$_->cannot_mklist()) {
-		  $_->togl();
-		}
-	 }
-  } else { 
-	 for (@{$this->{Objects}}) {
-		$_->togl()
-	 } 
-  }
-  print "EGENLIST $lno\n" if($PDL::Graphics::TriD::verbose);
-  #	pdltotrianglemesh($pdl, 0, 1, 0, ($pdl->{Dims}[1]-1)*$mult);
-  glEndList();
+  eval {
+    my @objs = @{$this->{Objects}};
+    @objs = grep !$_->cannot_mklist(), @objs if $PDL::Graphics::TriD::any_cannots;
+    $_->togl() for @objs;
+    print "EGENLIST $lno\n" if($PDL::Graphics::TriD::verbose);
+    #	pdltotrianglemesh($pdl, 0, 1, 0, ($pdl->{Dims}[1]-1)*$mult);
+  };
+  { local $@; glEndList(); }
+  die if $@;
   print "VALID1 $this\n" if($PDL::Graphics::TriD::verbose);
   $this->{ValidList} = 1;
 }
 
 sub PDL::Graphics::TriD::Object::gl_call_list {
 	my($this) = @_;
-	print "CALLIST ",$this->{List},"!\n" if($PDL::Graphics::TriD::verbose);
-	print "CHECKVALID $this\n" if($PDL::Graphics::TriD::verbose);
-
-	if(!$this->{ValidList}) {
-		$this->gl_update_list();
-	}
+	print "CALLIST ",$this->{List}//'undef',"!\n" if $PDL::Graphics::TriD::verbose;
+	print "CHECKVALID $this=$this->{ValidList}\n" if $PDL::Graphics::TriD::verbose;
+	$this->gl_update_list if !$this->{ValidList};
 	glCallList($this->{List});
-	if ($PDL::Graphics::TriD::any_cannots) {
-	  for(@{$this->{Objects}}) {
-		if($_->cannot_mklist()) {
-
-         print ref($_)," cannot mklist\n";
-
-			$_->togl();
-		}
-	  }
-        }
+	return if !$PDL::Graphics::TriD::any_cannots;
+	for (grep $_->cannot_mklist, @{$this->{Objects}}) {
+	  print ref($_)," cannot mklist\n";
+	  $_->togl();
+	}
 }
 
 sub PDL::Graphics::TriD::Object::delete_displist {
 	my($this) = @_;
-	if($this->{List}) {
-		glDeleteLists($this->{List},1);
-		undef $this->{List};
-	}
-}
-
-sub PDL::Graphics::TriD::Object::togl {
-	my($this) = @_;
-	for(@{$this->{Objects}}) { $_->togl() }
+	return if !$this->{List};
+	glDeleteLists($this->{List},1);
+	delete @$this{qw(List ValidList)};
 }
 
+sub PDL::Graphics::TriD::Object::togl { $_->togl for @{$_[0]->{Objects}} }
 
+my @bb1 = ([0,4,2],[0,1,2],[0,1,5],[0,4,5],[0,4,2],[3,4,2],
+	   [3,1,2],[3,1,5],[3,4,5],[3,4,2]);
+my @bb2 = ([0,1,2],[3,1,2],[0,1,5],[3,1,5],[0,4,5],[3,4,5]);
 sub PDL::Graphics::TriD::BoundingBox::togl { 
   my($this) = @_;
   $this = $this->{Box};
   glDisable(GL_LIGHTING);
   glColor3d(1,1,1);
   glBegin(GL_LINES);
-  for([0,4,2],[0,1,2],[0,1,5],[0,4,5],[0,4,2],[3,4,2],
-		[3,1,2],[3,1,5],[3,4,5],[3,4,2]) {
-	 &glVertex3d(@{$this}[@$_]);
-  }
+  glVertex3d(@{$this}[@$_]) for @bb1;
   glEnd();
   glBegin(GL_LINE_STRIP);
-  for([0,1,2],[3,1,2],[0,1,5],[3,1,5],[0,4,5],[3,4,5]) {
-	 &glVertex3d(@{$this}[@$_]);
-  }
+  glVertex3d(@{$this}[@$_]) for @bb2;
   glEnd();
   glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::Graph::togl {
 	my($this) = @_;
-#	print "TOGL Axis\n";
-	for(keys %{$this->{Axis}}) {
-		if($_ eq "Default") {next}
-		$this->{Axis}{$_}->togl_axis($this);
-	}
-#	print "TOGL DATA\n";
-	for(keys %{$this->{Data}}) {
-#	  print "TOGL   $_, $this->{Data}{$_}  $this->{Data}{$_}{Options}{LineWidth}\n";
-	  $this->{Data}{$_}->togl_graph($this,$this->get_points($_));
-	}
+	$this->{Axis}{$_}->togl_axis($this) for grep $_ ne "Default", keys %{$this->{Axis}};
+	$this->{Data}{$_}->togl_graph($this,$this->get_points($_)) for keys %{$this->{Data}};
 }
 
 use PDL;
 sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 	my($this,$graph) = @_;
-
 	my $fontbase = $PDL::Graphics::TriD::GL::fontbase;
-
-   
         my (@nadd,@nc,@ns);
-
 	for my $dim (0..1) {
 	  my $width = $this->{Scale}[$dim][1]-$this->{Scale}[$dim][0];
 	  if($width > 100){
@@ -150,19 +119,13 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 	  $nc[$dim] = int($this->{Scale}[$dim][0]/$nadd[$dim])*$nadd[$dim];
 	  $ns[$dim] = int($width/$nadd[$dim])+1;
 	}
-	
 	# can be changed to topo heights?
 	my $verts = zeroes(3,$ns[0],$ns[1]);
-
 	(my $t = $verts->slice("2")) .= 1012.5;
 	($t = $verts->slice("0")) .= $verts->ylinvals($nc[0],$nc[0]+$nadd[0]*($ns[0]-1));
 	($t = $verts->slice("1")) .= $verts->zlinvals($nc[1],$nc[1]+$nadd[1]*($ns[1]-1));
-
-
 	my $tverts = zeroes(3,$ns[0],$ns[1]);
-
 	$tverts = $this->transform($tverts,$verts,[0,1,2]);
-
 	glDisable(GL_LIGHTING);
 	glColor3d(1,1,1);
 	for(my $j=0;$j<$tverts->getdim(2)-1;$j++){
@@ -170,103 +133,55 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 	  glBegin(GL_LINES);
 	  for(my $i=0;$i<$tverts->getdim(1)-1;$i++){
 	    my $i1=$i+1;
-
 	    glVertex2f($tverts->at(0,$i,$j),$tverts->at(1,$i,$j));
 	    glVertex2f($tverts->at(0,$i1,$j),$tverts->at(1,$i1,$j));
-
 	    glVertex2f($tverts->at(0,$i1,$j),$tverts->at(1,$i1,$j));
 	    glVertex2f($tverts->at(0,$i1,$j1),$tverts->at(1,$i1,$j1));
-
 	    glVertex2f($tverts->at(0,$i1,$j1),$tverts->at(1,$i1,$j1));
 	    glVertex2f($tverts->at(0,$i,$j1),$tverts->at(1,$i,$j1));
-
 	    glVertex2f($tverts->at(0,$i,$j1),$tverts->at(1,$i,$j1));
 	    glVertex2f($tverts->at(0,$i,$j),$tverts->at(1,$i,$j));
-
 	  }
-
 	  glEnd();
 	}
-
 	glEnable(GL_LIGHTING);
 }
 
-
-
-
 sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 	my($this,$graph) = @_;
-
-        print "togl_axis: got object type " . ref($this) . "\n" if $PDL::debug_trid;
-#	print "TOGLAX\n";
+        print "togl_axis: got object type " . ref($this) . "\n" if $PDL::Graphics::TriD::verbose;
 	my $fontbase = $PDL::Graphics::TriD::GL::fontbase;
-#	print "TOGL EUCLID\n";
-   glLineWidth(1); # ought to be user defined
+	glLineWidth(1); # ought to be user defined
 	glDisable(GL_LIGHTING);
+	my $ndiv = 4;
+	my $line_coord = zeroes(3,3)->append(my $id3 = identity(3));
+	my $starts = zeroes($ndiv+1)->xlinvals(0,1)->transpose->append(zeroes(2,$ndiv+1));
+	my $ends = $starts + append(0, ones 2) * -0.1;
+	my $dupseq = sequence(3)->dummy(0,$ndiv+1)->flat;
+	$_ = $_->dup(1,3)->rotate($dupseq) for $starts, $ends;
+	$line_coord = $line_coord->glue(1, $starts->append($ends));
+	my $axisvals = zeroes(3,$ndiv+1)->ylinvals($this->{Scale}->dog)->transpose->flat->transpose;
+	my @label = map sprintf("%.3f", $_), @{ $axisvals->flat->unpdl };
+	push @label, @{$this->{Names}};
 	glColor3d(1,1,1);
-	glBegin(GL_LINES);
-	my $dim;
-	for $dim (0..2) {
-		glVertex3f(0,0,0);
-		&glVertex3f(map {$_==$dim} 0..2);
-	}
-	glEnd();
-	for $dim (0..2) {
-		my @coords = (0,0,0);
-		my @coords0 = (0,0,0);
-		for(0..2) {if($dim != $_) {
-				$coords[$_] -= 0.1;
-			}
-		}
-		my $s = $this->{Scale}[$dim];
-		my $ndiv = 3;
-		my $radd = 1.0/$ndiv;
-		my $nadd = ($s->[1]-$s->[0])/$ndiv;
-		my $nc = $s->[0];
-		for(0..$ndiv) {
-			&glRasterPos3f(@coords);
-			if ( OpenGL::done_glutInit() ) {
-			   OpenGL::glutBitmapString($fontbase, sprintf("%.3f",$nc));
-			} else {
-			   OpenGL::glpPrintString($fontbase, sprintf("%.3f",$nc));
-			}
-			glBegin(GL_LINES);
-			&glVertex3f(@coords0);
-			&glVertex3f(@coords);
-			glEnd();
-#			print "PUT: $nc\n";
-			$coords[$dim] += $radd;
-			$coords0[$dim] += $radd;
-			$nc += $nadd;
-		}
-		$coords0[$dim] = 1.1;
-		&glRasterPos3f(@coords0);
-		if ( OpenGL::done_glutInit() ) {
-			OpenGL::glutBitmapString($fontbase, $this->{Names}[$dim]);
-		} else {
-			OpenGL::glpPrintString($fontbase, $this->{Names}[$dim]);
-		}
-             }
+	PDL::Graphics::OpenGLQ::gl_texts($ends->glue(1, $id3), done_glutInit(),
+	    $fontbase, \@label);
+	PDL::gl_lines_nc($line_coord->splitdim(0,3)->clump(1,2));
 	glEnable(GL_LIGHTING);
 }
 
-
-
 use POSIX qw//;
 sub PDL::Graphics::TriD::Quaternion::togl {
   my($this) = @_;
   if(abs($this->[0]) == 1) { return ; }
   if(abs($this->[0]) >= 1) {
-    # die "Unnormalized Quaternion!\n";
     $this->normalize_this();
   } 
-  &glRotatef(2*POSIX::acos($this->[0])/3.14*180, @{$this}[1..3]);
+  glRotatef(2*POSIX::acos($this->[0])/3.14*180, @{$this}[1..3]);
 }
 
 ##################################
 # Graph Objects
-#
-#
 
 sub PDL::Graphics::TriD::GObject::togl {
 	$_[0]->gdraw($_[0]->{Points});
@@ -274,303 +189,262 @@ sub PDL::Graphics::TriD::GObject::togl {
 
 # (this,graphs,points)
 sub PDL::Graphics::TriD::GObject::togl_graph {
-#	print "TOGLGRAPH: $_[0]\n";
 	$_[0]->gdraw($_[2]);
 }
 
 sub PDL::Graphics::TriD::Points::gdraw {
 	my($this,$points) = @_;
-#	print "DRAWPOINTS: \n",$points;
-
-	$this->glOptions();
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_points($points,$this->{Colors});
-	glEnable(GL_LIGHTING);
-}
-
-sub PDL::gl_spheres { 
-   my ($coords,$colors) = @_;   
-   for (my $np=0; $np<$coords->dim(1); $np++) {
-      glPushMatrix();
-      my ($x,$y,$z) = ($coords->slice(":,($np)"))->float->list;
-      glTranslatef($x,$y,$z);
-      glutSolidSphere(0.025,15,15);
-      glPopMatrix();
-   }
+	eval {
+	  PDL::gl_points_col($points,$this->{Colors});
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::Spheres::gdraw {
    my($this,$points) = @_;
-   $this->glOptions();
+   glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+   $this->glOptions;
+   glEnable(GL_LIGHTING);
    glShadeModel(GL_SMOOTH);
-   PDL::gl_spheres($points,$this->{Colors});
+   eval {
+      PDL::gl_spheres($points, 0.025, 15, 15);
+   };
+   { local $@; glPopAttrib(); }
+   die if $@;
 }
 
 sub PDL::Graphics::TriD::Lattice::gdraw {
 	my($this,$points) = @_;
-
-	$this->glOptions();
+	barf "Need 3D points AND colours"
+	  if grep $_->ndims < 3, $points, $this->{Colors};
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_line_strip($points,$this->{Colors});
-	PDL::gl_line_strip($points->xchg(1,2),$this->{Colors}->xchg(1,2));
-	glEnable(GL_LIGHTING);
+	eval {
+	  PDL::gl_line_strip_col($points,$this->{Colors});
+	  PDL::gl_line_strip_col($points->xchg(1,2),$this->{Colors}->xchg(1,2));
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
-
 sub PDL::Graphics::TriD::LineStrip::gdraw {
 	my($this,$points) = @_;
-
-	$this->glOptions();
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_line_strip($points,$this->{Colors});
-	glEnable(GL_LIGHTING);
+	eval {
+	  PDL::gl_line_strip_col($points,$this->{Colors});
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::Lines::gdraw {
 	my($this,$points) = @_;
-
-	$this->glOptions();
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_lines($points,$this->{Colors});
-	glEnable(GL_LIGHTING);
+	eval {
+	  PDL::gl_lines_col($points,$this->{Colors});
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::GObject::glOptions {
   my ($this) = @_;
-
-  if($this->{Options}{LineWidth}){
-	 glLineWidth($this->{Options}{LineWidth});
-  }else{
-	 glLineWidth(1);
-  }
-
-  if($this->{Options}{PointSize}){
-	 glPointSize($this->{Options}{PointSize});
-  }else{
-	 glPointSize(1);
-  }
-  
-  
-
-
-
+  glLineWidth($this->{Options}{LineWidth} || 1);
+  glPointSize($this->{Options}{PointSize} || 1);
 }
 
+sub PDL::Graphics::TriD::GObject::_lattice_lines {
+  my ($this, $points) = @_;
+  glDisable(GL_LIGHTING);
+  glColor3f(0,0,0);
+  PDL::gl_line_strip_nc($points);
+  PDL::gl_line_strip_nc($points->xchg(1,2));
+}
 
 sub PDL::Graphics::TriD::Contours::gdraw {
-  my($this,$points) = @_;
-
-  $this->glOptions();
-
+  my ($this,$points) = @_;
+  glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+  $this->glOptions;
   glDisable(GL_LIGHTING);
-  my $pcnt=0;
-  my $i=0;
+  eval {
+    my $pi = $this->{PathIndex};
+    my ($pcnt, $i, $thisind) = (0, 0, 0);
+    for my $ie (grep defined, @{$this->{ContourPathIndexEnd}}) {
+      my $colors = $this->{Colors};
+      $colors = $colors->slice(":,($i)") if $colors->getndims==2;
+      my $this_pi = $pi->slice("$pcnt:$ie");
+      for ($this_pi->list) {
+        PDL::gl_line_strip_col($points->slice(",$thisind:$_"), $colors);
+        $thisind = $_ + 1;
+      }
+      $i++;
+      $pcnt=$ie+1;
+    }
+    if (defined $this->{Labels}){
+	   glColor3d(1,1,1);
+	   my $seg = sprintf ":,%d:%d",$this->{Labels}[0],$this->{Labels}[1];
+	   PDL::Graphics::OpenGLQ::gl_texts($points->slice($seg),
+		   done_glutInit(), $this->{Options}{Font},
+		   $this->{LabelStrings});
+    }
+  };
+  { local $@; glPopAttrib(); }
+  die if $@;
+}
 
-
-  foreach(@{$this->{ContourSegCnt}}){
-	 my $colors;
-	 if($this->{Colors}->getndims==2){
-		$colors = $this->{Colors}->slice(":,($i)");
-	 }else{
-		$colors =  $this->{Colors};
-	 }
-	 next unless(defined $_);
-	 PDL::gl_lines($points->slice(":,$pcnt:$_"),$colors);
-    $i++;
-	 $pcnt=$_+1;
+my @sls1 = (
+  ":,0:-2,0:-2",
+  ":,1:-1,0:-2",
+  ":,0:-2,1:-1");
+my @sls2 = (
+  ":,1:-1,1:-1",
+  ":,0:-2,1:-1",
+  ":,1:-1,0:-2");
+sub _lattice_slice {
+  my ($f, @pdls) = @_;
+  for my $s (\@sls1, \@sls2) {
+    my @args;
+    for my $p (@pdls) {
+      push @args, map $p->slice($_), @$s;
+    }
+    &$f(@args);
   }
-  if(defined $this->{Labels}){
-	 glColor3d(1,1,1);
-	 my $seg = sprintf(":,%d:%d",$this->{Labels}[0],$this->{Labels}[1]);
-	 PDL::Graphics::OpenGLQ::gl_texts($points->slice($seg),
-												 $this->{Options}{Font}
-												 ,$this->{LabelStrings});
-  }
-  
-  glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::SLattice::gdraw {
 	my($this,$points) = @_;
-
-	$this->glOptions();
-
+	barf "Need 3D points"
+	  if grep $_->ndims < 3, $points;
 	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (GL_SMOOTH);
-	my @sls1 = (":,0:-2,0:-2",
-	            ":,1:-1,0:-2",
-		    ":,0:-2,1:-1");
-	my @sls2 = (":,1:-1,1:-1",
-		    ":,0:-2,1:-1",
-	            ":,1:-1,0:-2"
-		    );
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls1),
-		(map {$this->{Colors}->slice($_)} @sls1)
-	);
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls2),
-		(map {$this->{Colors}->slice($_)} @sls2)
-	);
-	if ($this->{Options}{Lines}) {
-	  my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
-	  PDL::gl_line_strip($points,$black);
-	  PDL::gl_line_strip($points->xchg(1,2),$black);
-	}
-	glPopAttrib();
+	glShadeModel(GL_SMOOTH);
+	eval {
+	  _lattice_slice(\&PDL::gl_triangles, $points, $this->{Colors});
+	  $this->_lattice_lines($points) if $this->{Options}{Lines};
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::SCLattice::gdraw {
 	my($this,$points) = @_;
-
-	$this->glOptions();
-
+	barf "Need 3D points"
+	  if grep $_->ndims < 3, $points;
 	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (GL_FLAT);
-	my @sls1 = (":,0:-2,0:-2",
-	            ":,1:-1,0:-2",
-		    ":,0:-2,1:-1");
-	my @sls2 = (":,1:-1,1:-1",
-		    ":,0:-2,1:-1",
-	            ":,1:-1,0:-2"
-		    );
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls1),
-		(map {$this->{Colors}} @sls1)
-	);
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls2),
-		(map {$this->{Colors}} @sls2)
-	);
-	if ($this->{Options}{Lines}) {
-	  my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
-	  PDL::gl_line_strip($points,$black);
-	  PDL::gl_line_strip($points->xchg(1,2),$black);
-	}
-	glPopAttrib();
+	glShadeModel(GL_FLAT);
+	eval {
+	  _lattice_slice(\&PDL::gl_triangles, $points, $this->{Colors});
+	  $this->_lattice_lines($points) if $this->{Options}{Lines};
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::SLattice_S::gdraw {
 	my($this,$points) = @_;
-
-
-	$this->glOptions();
-
+	barf "Need 3D points"
+	  if grep $_->ndims < 3, $points;
 	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 # For some reason, we need to set this here as well.
 	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (GL_SMOOTH);
-	my @sls1 = (":,0:-2,0:-2",
-	            ":,1:-1,0:-2",
-		    ":,0:-2,1:-1");
-	my @sls2 = (":,1:-1,1:-1",
-		    ":,0:-2,1:-1",
-	            ":,1:-1,0:-2"
-		    );
-	if ($this->{Options}{Smooth}) {
-	  $this->{Normals} = $this->smoothn($points)
-	    unless defined($this->{Normals});
-	  my $n = $this->{Normals};
-	  my $f = (!$this->{Options}{Material} ?
-	  		\&PDL::gl_triangles_wn : \&PDL::gl_triangles_wn_mat);
-	  &$f(
-			       (map {$points->slice($_)} @sls1),
-			       (map {$n->slice($_)} @sls1),
-			       (map {$this->{Colors}->slice($_)} @sls1)
-			      );
-	  &$f(
-			       (map {$points->slice($_)} @sls2),
-			       (map {$n->slice($_)} @sls2),
-			       (map {$this->{Colors}->slice($_)} @sls2)
-			      );
-	} else {
-	  my $f = (!$this->{Options}{Material} ?
-	  		\&PDL::gl_triangles_n : \&PDL::gl_triangles_n_mat);
-	  &$f(
-			      (map {$points->slice($_)} @sls1),
-			      (map {$this->{Colors}->slice($_)} @sls1)
-			     );
-	  &$f(
-			      (map {$points->slice($_)} @sls2),
-			      (map {$this->{Colors}->slice($_)} @sls2)
-			     );
-	}
-	glDisable(GL_LIGHTING);
-	if ($this->{Options}{Lines}) {
-	  my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
-	  PDL::gl_line_strip($points,$black);
-	  PDL::gl_line_strip($points->xchg(1,2),$black);
-	}
-	glPopAttrib();
+	glShadeModel(GL_SMOOTH);
+	eval {
+	  my $f = 'PDL::gl_triangles_';
+	  $f .= 'w' if $this->{Options}{Smooth};
+	  $f .= 'n_mat';
+	  { no strict 'refs'; $f = \&$f; }
+	  my @pdls = $points;
+	  push @pdls, $this->{Normals} if $this->{Options}{Smooth};
+	  push @pdls, $this->{Colors};
+	  _lattice_slice($f, @pdls);
+	  $this->_lattice_lines($points) if $this->{Options}{Lines};
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
-#################################################################### 
-################### JNK 15mar11 added section start ################
 sub PDL::Graphics::TriD::STrigrid_S::gdraw {
   my($this,$points) = @_;
+  my $faces = $points->dice_axis(1,$this->{Faceidx}->flat)->splitdim(1,3);
   glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
-  # For some reason, we need to set this here as well.
-  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
-  # By-vertex doesn't make sense otherwise.
-  glShadeModel (GL_SMOOTH);   
-  my @sls = (":,(0)",":,(1)",":,(2)");
-  my $idx = [0,1,2,0]; # for lines, below
-  if ($this->{Options}{Smooth}) {
-    $this->{Normals}=$this->smoothn($this->{Points})
-      unless defined($this->{Normals}); 
-    my $f=(!$this->{Options}{Material}?\&PDL::gl_triangles_wn
-                                      :\&PDL::gl_triangles_wn_mat);
-    my $tmpn=$this->{Normals}->dice_axis(1,$this->{Faceidx}->clump(-1))
-                    ->splitdim(1,($this->{Faceidx}->dims)[0]);
-    my @args=((map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
-              (map {$tmpn->slice($_)} @sls),
-              (map {$this->{Colors}->slice($_)} @sls) );&$f(@args); }
-  else {
-    my $f=(!$this->{Options}{Material}?\&PDL::gl_triangles_n
-                                      :\&PDL::gl_triangles_n_mat);
-    &$f( (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
-         (map {$this->{Colors}->slice($_)} @sls) ); } 
-  glDisable(GL_LIGHTING);
-  if ($this->{Options}{Lines}) {
-    my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
-    PDL::gl_lines($this->{Faces}->dice_axis(1,$idx),$black); } glPopAttrib(); }
+  $this->glOptions;
+  eval {
+    # For some reason, we need to set this here as well.
+    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+    # By-vertex doesn't make sense otherwise.
+    glShadeModel(GL_SMOOTH);
+    my @sls = (":,(0)",":,(1)",":,(2)");
+    my $idx = [0,1,2,0]; # for lines, below
+    if ($this->{Options}{Smooth}) {
+      my $tmpn=$this->{Normals}->dice_axis(1,$this->{Faceidx}->flat)
+		      ->splitdim(1,$this->{Faceidx}->dim(0));
+      PDL::gl_triangles_wn_mat(map $_->mv(1,-1)->dog, $faces, $tmpn, $this->{Colors});
+      if ($this->{Options}{ShowNormals}) {
+	my $arrows = $points->append($points + $this->{Normals}*0.1)->splitdim(0,3);
+	glDisable(GL_LIGHTING);
+	glColor3d(1,1,1);
+	PDL::Graphics::OpenGLQ::gl_arrows($arrows, 0, 1, 0.5, 0.02);
+	my $facecentres = $faces->transpose->avgover;
+	my $facearrows = $facecentres->append($facecentres + $this->{FaceNormals}*0.1)->splitdim(0,3);
+	glColor3d(0.5,0.5,0.5);
+	PDL::Graphics::OpenGLQ::gl_arrows($facearrows, 0, 1, 0.5, 0.02);
+      }
+    } else {
+      PDL::gl_triangles_n_mat(map $_->mv(1,-1)->dog, $faces, $this->{Colors});
+    }
+    if ($this->{Options}{Lines}) {
+      glDisable(GL_LIGHTING);
+      glColor3f(0,0,0);
+      PDL::gl_lines_nc($this->{Faces}->dice_axis(1,$idx));
+    }
+  };
+  { local $@; glPopAttrib(); }
+  die if $@;
+}
 
 sub PDL::Graphics::TriD::STrigrid::gdraw {
   my($this,$points) = @_;
+  my $faces = $points->dice_axis(1,$this->{Faceidx}->flat)->splitdim(1,3);
+  # faces is 3D pdl slices of points, giving cart coords of face verts
   glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
-  glDisable(GL_LIGHTING);
+  $this->glOptions;
+  eval {
+    glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-  glShadeModel (GL_SMOOTH);
-  my @sls = (":,(0)",":,(1)",":,(2)");
-  my $idx = [0,1,2,0];
-  PDL::gl_triangles(
-    (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
-    (map {$this->{Colors}->slice($_)} @sls));
-  if ($this->{Options}{Lines}) {
-    my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
-    PDL::gl_lines($this->{Faces}->dice_axis(1,$idx),$black); }
-  glPopAttrib(); }
-################### JNK 15mar11 added section finis ################
-####################################################################
+    glShadeModel(GL_SMOOTH);
+    PDL::gl_triangles(map $_->mv(1,-1)->dog, $faces, $this->{Colors});
+    if ($this->{Options}{Lines}) {
+      glColor3f(0,0,0);
+      PDL::gl_lines_nc($faces->dice_axis(1, [0,1,2,0]));
+    }
+  };
+  { local $@; glPopAttrib(); }
+  die if $@;
+}
 
 ##################################
 # PDL::Graphics::TriD::Image
-#
-#
 
 sub PDL::Graphics::TriD::Image::togl {
-#  glDisable(GL_LIGHTING);
-#
 # A special construct which always faces the display and takes the entire window
-#  
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
-
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0,1,0,1);
@@ -581,79 +455,69 @@ sub PDL::Graphics::TriD::Image::togl_graph {
 	$_[0]->gdraw();
 }
 
-
 # The quick method is to use texturing for the good effect.
 sub PDL::Graphics::TriD::Image::gdraw {
 	my($this,$vert) = @_;
 	my ($p,$xd,$yd,$txd,$tyd) = $this->flatten(1); # do binary alignment
+	if(!defined $vert) {$vert = $this->{Points}}
+	barf "Need 3,4 vert"
+	  if grep $_->dim(1) < 4 || $_->dim(0) != 3, $vert;
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
 	glColor3d(1,1,1);
          glTexImage2D_s(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
 	 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
 	    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
 	       glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 	          glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
-
 	glDisable(GL_LIGHTING);
 	glNormal3d(0,0,1);
 	glEnable(GL_TEXTURE_2D);
 	glBegin(GL_QUADS);
-	my @texvert = (
-		[0,0],
-		[$xd/$txd, 0],
-		[$xd/$txd, $yd/$tyd],
-		[0, $yd/$tyd]
-	);
-	if(!defined $vert) {$vert = $this->{Points}}
-	for(0..3) {
-		&glTexCoord2f(@{$texvert[$_]});
-		&glVertex3f($vert->slice(":,($_)")->list);
-	}
-	glEnd();
-	glEnable(GL_LIGHTING);
-	glDisable(GL_TEXTURE_2D);
+	eval {
+	  my @texvert = (
+		  [0,0],
+		  [$xd/$txd, 0],
+		  [$xd/$txd, $yd/$tyd],
+		  [0, $yd/$tyd]
+	  );
+	  for(0..3) {
+		  glTexCoord2f(@{$texvert[$_]});
+		  glVertex3f($vert->slice(":,($_)")->list);
+	  }
+	};
+	{ local $@; glEnd(); glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::SimpleController::togl {
 	my($this) = @_;
-
 	$this->{CRotation}->togl();
 	glTranslatef(0,0,-$this->{CDistance});
-
 	$this->{WRotation}->togl();
-	&glTranslatef(map {-$_} @{$this->{WOrigin}});
+	glTranslatef(map {-$_} @{$this->{WOrigin}});
 }
 
-
 ##############################################
-#
 # A window with mouse control over rotation.
-#
-#
 package PDL::Graphics::TriD::Window;
 
-use OpenGL qw(:all);
+use OpenGL qw/ :glfunctions :glconstants :glxconstants /;
+use OpenGL::GLUT qw( :all );
 use PDL::Graphics::OpenGL::Perl::OpenGL;
 
 use base qw/PDL::Graphics::TriD::Object/;
-use fields qw/Ev Width Height Interactive _GLObject 
+use fields qw/Ev Width Height Interactive _GLObject
               _ViewPorts _CurrentViewPort /;
 
-sub i_keep_list {return 1} # For Object, so I will be notified of changes.
-use strict;
-
 sub gdriver {
   my($this, $options) = @_;
-  
-  print "GL gdriver...\n" if($PDL::debug_trid);
-
+  print "GL gdriver...\n" if($PDL::Graphics::TriD::verbose);
   if(defined $this->{_GLObject}){
 	 print "WARNING: Graphics Driver already defined for this window \n";
 	 return;
   }
-  my @db = GLX_DOUBLEBUFFER;
-
+  my @db = OpenGL::GLX_DOUBLEBUFFER;
   if($PDL::Graphics::TriD::offline) {$options->{x} = -1; @db=()}
-
   $options->{attributes} = [GLX_RGBA, @db,
 			    GLX_RED_SIZE,1,
 			    GLX_GREEN_SIZE,1,
@@ -661,76 +525,36 @@ sub gdriver {
 			    GLX_DEPTH_SIZE,1,
 			    # Alpha size?
 			   ] unless defined $options->{attributes};
-    
   $options->{mask} = (KeyPressMask | ButtonPressMask |
 			 ButtonMotionMask | ButtonReleaseMask |
 			 ExposureMask | StructureNotifyMask |
 			 PointerMotionMask) unless defined $options->{mask};
-
-
   print "STARTING OPENGL $options->{width} $options->{height}\n" if($PDL::Graphics::TriD::verbose);
-
-  print "gdriver: Calling OpengGL::OO($options)...\n" if ($PDL::debug_trid);
-
-  $this->{_GLObject}= new PDL::Graphics::OpenGL::OO($options);
-
+  print "gdriver: Calling OpengGL::OO($options)...\n" if ($PDL::Graphics::TriD::verbose);
+  $this->{_GLObject}= PDL::Graphics::OpenGL::OO->new($options);
   if (exists $this->{_GLObject}->{glutwindow}) {
-     if ($PDL::debug_trid) {
+     if ($PDL::Graphics::TriD::verbose) {
         print "gdriver: Got OpenGL::OO object(GLUT window ID# " . $this->{_GLObject}->{glutwindow} . ")\n";
      }
      $this->{_GLObject}->{winobjects}->[$this->{_GLObject}->{glutwindow}] = $this;      # circular ref
   }
-
-#glpOpenWindow(%$options);
-  
-  print "gdriver: Calling glClearColor...\n" if ($PDL::debug_trid);
+  print "gdriver: Calling glClearColor...\n" if $PDL::Graphics::TriD::verbose;
   glClearColor(0,0,0,1);
-
-  print "gdriver: Calling glpRasterFont...\n" if ($PDL::debug_trid);
-  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
-     print STDERR "gdriver: window_type => 'glut' so not actually setting the rasterfont\n" if ($PDL::debug_trid);
-     eval '$PDL::Graphics::TriD::GL::fontbase = GLUT_BITMAP_8_BY_13';
-  } else {
-     # NOTE: glpRasterFont() will die() if the requested font cannot be found
-     #       The new POGL+GLUT TriD implementation uses the builtin GLUT defined
-     #       fonts and does not have this failure mode.
-     
-     my $lb =  eval { $this->{_GLObject}->glpRasterFont( ($ENV{PDL_3D_FONT} or "5x8"), 0, 256 ) };
-     if ( $@ ) {
-        die "glpRasterFont: unable to load font '%s', please set PDL_3D_FONT to an existing X11 font.";
-     }
-     $PDL::Graphics::TriD::GL::fontbase = $lb
-  }
-  #	glDisable(GL_DITHER);
-  glShadeModel (GL_FLAT);
+  print "gdriver: Calling glpRasterFont...\n" if $PDL::Graphics::TriD::verbose;
+  $PDL::Graphics::TriD::GL::fontbase = $this->{_GLObject}->glpRasterFont($ENV{PDL_3D_FONT} || "5x8", 0, 256);
+  glShadeModel(GL_FLAT);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_NORMALIZE);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
-  
-  # Will this bring us trouble?
-  #	if(defined *PDL::Graphics::TriD::GL::Window::glPolygonOffsetEXT{CODE}) {
-#  glEnable(GL_POLYGON_OFFSET_EXT);
-#  glPolygonOffsetEXT(0.0000000000001,0.000002);
-  #	}
-  
-  # Inherits attributes of Object class
-#  my $this = $type->SUPER::new();
-#  $this->reshape($options->{width},$options->{height});
-
   my $light = pack "f*",1.0,1.0,1.0,0.0;
   glLightfv_s(GL_LIGHT0,GL_POSITION,$light);
-
   glColor3f(1,1,1);
-  
-#  $this->{Interactive} = 1;
-  print "STARTED OPENGL!\n" if($PDL::Graphics::TriD::verbose);
-  
+  print "STARTED OPENGL!\n" if $PDL::Graphics::TriD::verbose;
   if($PDL::Graphics::TriD::offline) {
     $this->doconfig($options->{width}, $options->{height});
   }
-  
   return 1;  # Interactive Window
 }
 
@@ -745,7 +569,6 @@ sub reshape {
 	my $pw = $this->{Width};
 	my $ph = $this->{Height};
 	$this->{Width} = $x; $this->{Height} = $y;
-
 	for my $vp (@{$this->{_ViewPorts}}){
 	  my $nw = $vp->{W} + ($x-$pw) * $vp->{W}/$pw;
 	  my $nx0 = $vp->{X0} + ($x-$pw) * $vp->{X0}/$pw;
@@ -754,16 +577,9 @@ sub reshape {
 	  print "reshape: resizing viewport to $nx0,$ny0,$nw,$nh\n" if($PDL::Graphics::TriD::verbose);
 	  $vp->resize($nx0,$ny0,$nw,$nh);
 	}
-
-}
-
-
-
-sub get_size {
-  my $this=shift;
-  return ($this->{Width},$this->{Height});
 }
 
+sub get_size { @{$_[0]}{qw(Width Height)} }
 
 sub twiddle {
   my($this,$getout,$dontshow) = @_;
@@ -776,33 +592,18 @@ sub twiddle {
 	 wpic($this->read_picture(),"PDL_$PDL::Graphics::TriD::offlineindex.jpg");
 	 return;
   }
-  if ($getout and $dontshow) {
-	 if ( !$this->{_GLObject}->XPending() ) {
-            return;
-         }
-  }
-  if(!defined $getout) {
-	 $getout = not ($PDL::Graphics::TriD::keeptwiddling && $PDL::Graphics::TriD::keeptwiddling);
-  }
-  
+  return if $getout and $dontshow and !$this->{_GLObject}->XPending;
+  $getout //= !($PDL::Graphics::TriD::keeptwiddling && $PDL::Graphics::TriD::keeptwiddling);
   $this->display();
  TWIDLOOP: while(1) {
    print "EVENT!\n" if($PDL::Graphics::TriD::verbose);
 	 my $hap = 0;
 	 my $gotev = 0;
-
-         # Run a MainLoop event if GLUT windows
-         # this pumps the system allowing callbacks to populate
-         # the fake XEvent queue.
-         #
-         glutMainLoopEvent() if $this->{_GLObject}->{window_type} eq 'glut' and not $this->{_GLObject}->XPending();
-
          if ($this->{_GLObject}->XPending() or !$getout) {
             @e = $this->{_GLObject}->glpXNextEvent();
             $gotev=1;
          }
    print "e= ".join(",",@e)."\n" if($PDL::Graphics::TriD::verbose);
-	
 	 if(@e){
 		if ($e[0] == VisibilityNotify || $e[0] == Expose) {
 		  $hap = 1;
@@ -810,6 +611,12 @@ sub twiddle {
 		  print "CONFIGNOTIFE\n" if($PDL::Graphics::TriD::verbose);
 		  $this->reshape($e[1],$e[2]);
 		  $hap=1;
+		} elsif ($e[0] == DestroyNotify) {
+		  print "DESTROYNOTIFE\n" if $PDL::Graphics::TriD::verbose;
+		  $quit = 1;
+		  $hap=1;
+		  $this->close;
+		  last TWIDLOOP;
 		} elsif($e[0] == KeyPress) {
 		  print "KEYPRESS: '$e[1]'\n" if($PDL::Graphics::TriD::verbose);
 		  if((lc $e[1]) eq "q") {
@@ -824,12 +631,11 @@ sub twiddle {
 		  $hap=1;
 		}
 	 }
-
 	 if($gotev){
 		#			print "HANDLING $this->{EHandler}\n";
 		foreach my $vp (@{$this->{_ViewPorts}}) {
 		  if(defined($vp->{EHandler})) {
-			 $hap += $vp->{EHandler}->event(@e);
+			 $hap += $vp->{EHandler}->event(@e) || 0;
 		  }
 		}
 	 }
@@ -839,18 +645,21 @@ sub twiddle {
 		}
 		if($getout) {last TWIDLOOP}
 	 }
-	 undef @e;
+	 @e = ();
   }
   print "STOPTWIDDLE\n" if($PDL::Graphics::TriD::verbose);
   return $quit;
 }
 
-
-
+sub close {
+  my ($this, $close_window) = @_;
+  print "CLOSE\n" if $PDL::Graphics::TriD::verbose;
+  undef $this->{_GLObject};
+  $PDL::Graphics::TriD::current_window = undef;
+}
 
 sub setlist { my($this,$list) = @_;
 	$this->{List} = $list;
-
 }
 
 # Resize window.
@@ -867,53 +676,26 @@ sub domotion {
 
 sub display {
   my($this) = @_;
-
   return unless defined($this);
-
-  # set GLUT context to current window (for multiwindow support
-  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
-     glutSetWindow($this->{_GLObject}->{glutwindow});
-  }
-
+  $this->{_GLObject}->set_window; # for multiwindow support
   print "display: calling glClear()\n" if ($PDL::Graphics::TriD::verbose);
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   glMatrixMode(GL_MODELVIEW);
-
-
   for my $vp (@{$this->{_ViewPorts}}) {
-
 	 glPushMatrix();
-    
-
 	 $vp->do_perspective();
-
-
 	 if($vp->{Transformer}) {
 		print "display: transforming viewport!\n" if ($PDL::Graphics::TriD::verbose);
 		$vp->{Transformer}->togl();
 	 }
-
-	 glTranslatef(-1,-1,-1);
-	 glScalef(2,2,2);  # double the scale in each direction ?
-
 	 $vp->gl_call_list();
-
 	 glPopMatrix();
-
   }
-
-  print "display: SwapBuffers() call on return\n" if ($PDL::Graphics::TriD::verbose);
-  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {  # need to make method call
-    glutSwapBuffers();
-  } elsif ( $this->{_GLObject}->{window_type} eq 'x11' ) {  # need to make method call
-    $this->{_GLObject}->glXSwapBuffers();
-  } else {
-    print "display: got object with inconsistent _GLObject info\n";
-  }
-#  $this->{Angle}+= 3;
+  $this->{_GLObject}->swap_buffers;
+  print "display: after SwapBuffers\n" if $PDL::Graphics::TriD::verbose;
 }
 
-# should this reallyt be in viewport?
+# should this really be in viewport?
 sub read_picture {
 	my($this) = @_;
 	my($w,$h) = @{$this}{qw/Width Height/};
@@ -930,41 +712,38 @@ sub read_picture {
 
 package PDL::Graphics::TriD::EventHandler;
 
-use OpenGL qw(ConfigureNotify MotionNotify ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask);
+use OpenGL qw(
+  ConfigureNotify MotionNotify DestroyNotify
+  ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask Button4Mask
+);
 use PDL::Graphics::OpenGL::Perl::OpenGL;
 
 use fields qw/X Y Buttons VP/;
-use strict;
 sub new {
   my $class = shift;
   my $vp = shift;
-  no strict 'refs';
   my $self = fields::new($class);
   $self->{X} = -1;
   $self->{Y} = -1;
   $self->{Buttons} = [];
   $self->{VP} = $vp;
-
   $self;
 }
 
 sub event {
   my($this,$type,@args) = @_;
-
   print "EH: ",ref($this)," $type (",join(",",@args),")\n" if($PDL::Graphics::TriD::verbose);
   my $retval;
-
   if($type == MotionNotify) {
 	 my $but = -1;
-	 
-  SWITCH: { 
+  SWITCH: {
 		$but = 0, last SWITCH if ($args[0] & (Button1Mask));
 		$but = 1, last SWITCH if ($args[0] & (Button2Mask));
 		$but = 2, last SWITCH if ($args[0] & (Button3Mask));
+		$but = 3, last SWITCH if ($args[0] & (Button4Mask));
 		print "No button pressed...\n" if($PDL::Graphics::TriD::verbose);
 		goto NOBUT;
 	 }
-
 	 print "MOTION $but $args[0]\n" if($PDL::Graphics::TriD::verbose);
 	 if($this->{Buttons}[$but]) {
 		if($this->{VP}->{Active}){
@@ -976,29 +755,21 @@ sub event {
 	 }
 	 $this->{X} = $args[1]; $this->{Y} = $args[2];
   NOBUT:
-
        } elsif($type == ButtonPress) {
-
 	 my $but = $args[0]-1;
 	 print "BUTTONPRESS $but\n" if($PDL::Graphics::TriD::verbose);
 	 $this->{X} = $args[1]; $this->{Y} = $args[2];
-	 $retval = $this->{Buttons}[$but]->ButtonPress($args[1],$args[2]) 
+	 $retval = $this->{Buttons}[$but]->ButtonPress($args[1],$args[2])
 	   if($this->{Buttons}[$but]);
-
        } elsif($type == ButtonRelease) {
-
 	 my $but = $args[0]-1;
 	 print "BUTTONRELEASE $but\n" if($PDL::Graphics::TriD::verbose);
-	 $retval = $this->{Buttons}[$but]->ButtonRelease($args[1],$args[2]) 
+	 $retval = $this->{Buttons}[$but]->ButtonRelease($args[1],$args[2])
 	   if($this->{Buttons}[$but]);
-
        } elsif($type== ConfigureNotify) {
-
 	 # Kludge to force reshape of the viewport associated with the window -CD
 	 print "ConfigureNotify (".join(",",@args).")\n" if($PDL::Graphics::TriD::verbose);
 	 print "viewport is $this->{VP}\n" if($PDL::Graphics::TriD::verbose);
-#	 $retval = $this->reshape(@args);
-
        }
   $retval;
 }
@@ -1008,7 +779,6 @@ sub set_button {
 	$this->{Buttons}[$butno] = $act;
 }
 
-  
 ######################################################################
 ######################################################################
 # VIEWPORT MINI_PACKAGE FOLLOWS!
@@ -1018,76 +788,46 @@ use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/X0 Y0 W H Transformer EHandler Active ResizeCommands 
               DefMaterial AspectRatio Graphs/;
 
-use OpenGL qw(:all);
+use OpenGL qw/ :glfunctions :glconstants :glufunctions /;
+use OpenGL::GLUT qw( :all );
 use PDL::Graphics::OpenGL::Perl::OpenGL;
 use PDL::Graphics::OpenGLQ;
 
 sub highlight {
   my ($vp) = @_;
-
-  my $pts =  new PDL [[0,0,0],
-				  [$vp->{W},0,0],
-				  [$vp->{W},$vp->{H},0],
-				  [0,$vp->{H},0],
-				  [0,0,0]];
-  my $colors;
-
-  $colors = PDL->ones(3,5);
-
+  my $pts = PDL->new([[0,0,0],
+		      [$vp->{W},0,0],
+		      [$vp->{W},$vp->{H},0],
+		      [0,$vp->{H},0],
+		      [0,0,0]]);
   glDisable(GL_LIGHTING);
-  
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
-
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0,$vp->{W},0,$vp->{H});
   glLineWidth(4);
-  
-  gl_line_strip($pts,$colors);
-
+  glColor3f(1,1,1);
+  gl_line_strip_nc($pts);
   glLineWidth(1);
-		
   glEnable(GL_LIGHTING);
-
 }
 
-
-
 sub do_perspective {
 	my($this) = @_;
-
-	print "do_perspective ",$this->{W}," ",$this->{H} ,"\n" if($PDL::Graphics::TriD::verbose);
-
-	if($PDL::Graphics::TriD::verbose>1){
-	  my ($i,$package,$filename,$line);
-          $i = 0;
-	  do { 
-	    ($package,$filename,$line) = caller($i++);
-	    print "$package ($filename, line $line)\n";
-	  } while($package);
-	  print "\n";
-	}
-	      
-
+	print "do_perspective ",$this->{W}," ",$this->{H} ,"\n" if $PDL::Graphics::TriD::verbose;
+	print Carp::longmess() if $PDL::Graphics::TriD::verbose>1;
         unless($this->{W}>0 and $this->{H}>0) {return;}
-#	if($this->{W}==0 or $this->{H}==0) {return;}
 	$this->{AspectRatio} = (1.0*$this->{W})/$this->{H};
-#	glResizeBuffers();
-
 	glViewport($this->{X0},$this->{Y0},$this->{W},$this->{H});
-
-	$this->highlight() if($this->{Active});
-
+	$this->highlight if $this->{Active};
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-
 	gluPerspective(40.0, $this->{AspectRatio} , 0.1, 200000.0);
 	glMatrixMode(GL_MODELVIEW);
 	glLoadIdentity ();
 }
 
-
 ###############
 #
 # Because of the way GL does texturing, this must be the very last thing
@@ -1115,6 +855,4 @@ sub PDL::Graphics::TriD::GL::SliceTexture::togl {
 	glDisable(GL_TEXTURE_1D);
 }
 
-
-
 1;
