@@ -144,7 +144,7 @@ C
       implicit integer*8(i-n)
       INTEGER*8  N, INCFD, NE, IERR
       REAL  X(*), F(INCFD,*), D(INCFD,*), XE(*), FE(*), DE(*)
-      LOGICAL  SKIP
+      INTEGER*4  SKIP
 C
 C  DECLARE LOCAL VARIABLES.
 C
@@ -153,7 +153,7 @@ C
 C  VALIDITY-CHECK ARGUMENTS.
 C
 C***FIRST EXECUTABLE STATEMENT  PCHFD
-      IF (SKIP)  GO TO 5
+      IF (SKIP.NE.0)  GO TO 5
 C
       IF ( N.LT.2 )  GO TO 5001
       IF ( INCFD.LT.1 )  GO TO 5002
